#include <assert.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <fcntl.h>
#include <linux/fb.h>
#include <sys/mman.h>
#include "fbutil.h"

int fb_open(char *devname, struct fbinfo *fbi) {
    unsigned long screensize;
    int st;
    /* Open the file for reading and writing */
    fbi->fd = open(devname, O_RDWR);
    assert(fbi->fd);
    /* Get fixed screen information */
    st = ioctl(fbi->fd, FBIOGET_FSCREENINFO, &fbi->finfo);
    assert(!st);
    assert(fbi->finfo.type == FB_TYPE_PACKED_PIXELS);
    assert(fbi->finfo.visual == FB_VISUAL_TRUECOLOR);
    /* Get variable screen information */
    st = ioctl(fbi->fd, FBIOGET_VSCREENINFO, &fbi->vinfo);
    assert(!st);
    assert(fbi->vinfo.bits_per_pixel == 16);
    assert(!fbi->vinfo.grayscale);
    /* 565 color */
    assert(fbi->vinfo.red.offset == 11);
    assert(fbi->vinfo.red.length == 5);
    assert(!fbi->vinfo.red.msb_right);
    assert(fbi->vinfo.green.offset == 5);
    assert(fbi->vinfo.green.length == 6);
    assert(!fbi->vinfo.green.msb_right);
    assert(fbi->vinfo.blue.offset == 0);
    assert(fbi->vinfo.blue.length == 5);
    assert(!fbi->vinfo.blue.msb_right);
    assert(fbi->vinfo.transp.length == 0); /* no alpha */
    /* Figure out the size of the screen in bytes */
    screensize =
	fbi->vinfo.xres * fbi->vinfo.yres * fbi->vinfo.bits_per_pixel / 8;
    /* Map the device to memory */
    fbi->map = mmap(0, screensize, PROT_READ | PROT_WRITE, MAP_SHARED,
		   fbi->fd, 0);
    assert(fbi->map != MAP_FAILED);
    /* success! */
    return 0;
}

void fb_put(struct fbinfo *fbi, int xoff, int yoff, struct image *image) {
    int y;
    /* copy the pic */
    for (y=0; y < image->height; y++) {
	int len, x0, x1, sx0, sx1, sy;
	sy = y + yoff;
	if (sy < 0 || sy >= fbi->vinfo.yres) continue;
	/* memcpy a full row */
	x0 = 0; sx0 = x0 + xoff;
	x1 = image->width; sx1 = x1 + xoff;
	if (sx0 < 0) { x0 -= sx0; sx0 = 0; }
	if (sx1 > fbi->vinfo.xres) { int diff = sx1 - fbi->vinfo.xres; x1 -= diff; sx1 -= diff; }
	memcpy(fbi->map +
	       (sx0 + fbi->vinfo.xoffset) * (fbi->vinfo.bits_per_pixel/8) +
	       (sy + fbi->vinfo.yoffset) * fbi->finfo.line_length,
	       image->pixel_data + x0 + y * image->width,
	       sizeof(uint16_t) * (x1 - x0));
    }
}

void fb_close(struct fbinfo *fbi) {
    unsigned long screensize;
    screensize =
	fbi->vinfo.xres * fbi->vinfo.yres * fbi->vinfo.bits_per_pixel / 8;
    munmap(fbi->map, screensize);
    close(fbi->fd);
}
